@@ -934,6 +934,29 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_CRL_VERIFY)) {
+			const char *file;
+			gs_free char *file_free = NULL;
+
+			if (!args_params_check_nargs_minmax (params, 1, 2, &line_error))
+				goto handle_line_error;
+			if (!args_params_check_arg_nonempty (params, 1, NULL, &line_error))
+				goto handle_line_error;
+			if (params[2] && !nm_streq (params[2], "dir")) {
+				line_error = g_strdup_printf (_("unsupported crl-verify argument"));
+				goto handle_line_error;
+			}
+
+			file = params[1];
+			if (!g_path_is_absolute (file))
+				file = file_free = g_build_filename (default_path, file, NULL);
+
+			if (params[2])
+				setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_CRL_VERIFY_DIR, file);
+			else
+				setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_CRL_VERIFY_FILE, file);
+		}
+
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_NS_CERT_TYPE)) {
 			if (!args_params_check_nargs_n (params, 1, &line_error))
 				goto handle_line_error;
@@ -1401,7 +1424,7 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			}
 
 			{
-#ifdef NM_VPN_OLD
+#if ((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_UTIL)
 				NMIP4Route *route;
 
 				route = nm_ip4_route_new ();
@@ -1652,7 +1675,7 @@ escape_arg (const char *value, char **buf)
 {
 	const char *s;
 	char *result, *i_result;
-	gboolean has_single_quote = FALSE;
+	gboolean needs_double_quotes = FALSE;
 	gboolean needs_quotation = FALSE;
 	gsize len;
 
@@ -1673,14 +1696,15 @@ escape_arg (const char *value, char **buf)
 		    || (c >= 'A' && c <= 'Z')
 		    || NM_IN_SET (c, '_', '-', ':', '/'))
 			continue;
+
 		needs_quotation = TRUE;
-		if (c == '\'')
-			has_single_quote = TRUE;
+		if (NM_IN_SET (c, '\'', '\n'))
+			needs_double_quotes = TRUE;
 	}
 	if (!needs_quotation)
 		return value;
 
-	if (!has_single_quote) {
+	if (!needs_double_quotes) {
 		result = g_malloc (len + 2 + 1);
 		result[0] = '\'';
 		memcpy (&result[1], value, len);
@@ -1690,9 +1714,19 @@ escape_arg (const char *value, char **buf)
 		i_result = result = g_malloc (len * 2 + 3);
 		*(i_result++) = '"';
 		for (s = value; s[0]; s++) {
-			if (NM_IN_SET (s[0], '\\', '"'))
+			if (s[0] == '\n') {
+				/* Openvpn does not support encoding '\n' in the ovpn configuration file.
+				 * This configuration cannot be expressed in an ovpn file.
+				 *
+				 * Still escape it as '\n', although openvpn's parser will warn
+				 * about the invalid escape sequence. */
 				*(i_result++) = '\\';
-			*(i_result++) = s[0];
+				*(i_result++) = 'n';
+			} else {
+				if (NM_IN_SET (s[0], '\\', '"'))
+					*(i_result++) = '\\';
+				*(i_result++) = s[0];
+			}
 		}
 		*(i_result++) = '"';
 		*(i_result++) = '\0';
@@ -1939,6 +1973,15 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	args_write_line_setting_value_int (f, NMV_OVPN_TAG_FRAGMENT, s_vpn, NM_OPENVPN_KEY_FRAGMENT_SIZE);
 
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CRL_VERIFY_FILE);
+	if (value)
+		args_write_line (f, NMV_OVPN_TAG_CRL_VERIFY, value);
+	else {
+		value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CRL_VERIFY_DIR);
+		if (value)
+			args_write_line (f, NMV_OVPN_TAG_CRL_VERIFY, value, "dir");
+	}
+
 	{
 		gs_free char *device_free = NULL;
 		const char *device_type, *device;
@@ -2079,7 +2122,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 
 	s_ip4 = nm_connection_get_setting_ip4_config (connection);
 	if (s_ip4) {
-#ifdef NM_VPN_OLD
+#if ((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_UTIL)
 		num = nm_setting_ip4_config_get_num_routes (s_ip4);
 #else
 		num = nm_setting_ip_config_get_num_routes (s_ip4);
@@ -2092,7 +2135,7 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 			guint64 metric;
 			char metric_buf[50];
 
-#ifdef NM_VPN_OLD
+#if ((NETWORKMANAGER_COMPILATION) & NM_NETWORKMANAGER_COMPILATION_WITH_LIBNM_UTIL)
 			char next_hop_str_buf[INET_ADDRSTRLEN] = { 0 };
 			char dest_str_buf[INET_ADDRSTRLEN] = { 0 };
 			in_addr_t dest, next_hop;
